function [x] = IDTWPT_JJW(y,h_first,h,f,best_basis_mask)
% function to perform inverse wavelet packet transform
% adapted by JJW to take the best basis mask and use this to determine the
% tree for reconstruction, rather than just reconstructing from all the
% packets at the highest scale
%full packet inverse transform
%y : the cell array arranged as in DTWPT
%h_first : first stage synthesis filters([h0_first;h1_first])
%h : dual-tree synthesis filters([h0;h1])
%f : the 'same' synthesis filters([f0;f1])
% Adapted by Jez Wells, University of York (jjw100@ohm.york.ac.uk) from
% code by I. Bayram available at http://www2.itu.edu.tr/~ibayram/dtcwpt/

max_level = ceil(log2(size(y,2)));

xx = y(max_level,:);

for n = max_level:-1:3,
    for k=1:2^(n-1)
        if(best_basis_mask(n,2*k))
            % filter and place coefficients in the next scale up
            if mod(k,2^(n-2))==1,
                fil0=h(1,:);
                fil1=h(2,:);
            else
                fil0=f(1,:);
                fil1=f(2,:);
            end
            if(length(xx{2*k-1})>length(xx{2*k}))
                % append with zeros
                xx{2*k} = [xx{2*k},zeros(1,length(xx{2*k-1})-length(xx{2*k}))];
            elseif(length(xx{2*k-1})<length(xx{2*k}))
                % append with zeros
                xx{2*k-1} = [xx{2*k-1},zeros(1,length(xx{2*k})-length(xx{2*k-1}))];
            end
            x2{k} = sfb([xx{2*k-1};xx{2*k}],fil0,fil1);
        else
            % do nothing, keep existing coefficients in the next scale up
            x2{k} = y{n-1,k};
        end
    end
    xx=x2;
end

%second stage
fil0=h(1,:);
fil1=h(2,:);
if(best_basis_mask(2,1))
    if(length(xx{1})>length(xx{2}))
        % append with zeros
        xx{2} = [xx{2},zeros(1,length(xx{1})-length(xx{2}))];
    elseif(length(xx{1})<length(xx{2}))
        % append with zeros
        xx{1} = [xx{1},zeros(1,length(xx{2})-length(xx{1}))];
    end
    x2{1}=sfb([xx{1};xx{2}],fil0,fil1);
else
    % do nothing, keep existing coefficients in the next scale up
    x2{1} = y{1,1};
end
if(best_basis_mask(2,3))
    if(length(xx{3})>length(xx{4}))
        % append with zeros
        xx{4} = [xx{4},zeros(1,length(xx{3})-length(xx{4}))];
    elseif(length(xx{3})<length(xx{4}))
        % append with zeros
        xx{3} = [xx{3},zeros(1,length(xx{4})-length(xx{3}))];
    end
    x2{2}=sfb([xx{3};xx{4}],fil0,fil1);
else
    % do nothing, keep existing coefficients in the next scale up
    x2{2} = y{1,2};
end
    

%first stage
fil0=h_first(1,:);
fil1=h_first(2,:);
if(length(x2{1})>length(x2{2}))
    % append with zeros
    x2{2} = [x2{2},zeros(1,length(x2{1})-length(x2{2}))];
elseif(length(x2{1})<length(x2{2}))
    % append with zeros
    x2{1} = [x2{1},zeros(1,length(x2{2})-length(x2{1}))];
end
x=sfb([x2{1};x2{2}],fil0,fil1);